/*
** It's an analog world
** Converting the analog signal from a potentiometer
*/

#include <p24Fxxxx.h>

#if defined( __PIC24FJ256GB110__ ) || defined( __PIC24FJ256GB106__ )
#define PLL_96MHZ_ON    0xF7FF
_CONFIG1(ICS_PGx2 & JTAGEN_OFF & FWDTEN_OFF)        // JTAG off, watchdog timer off
_CONFIG2( FNOSC_PRIPLL & PLL_96MHZ_ON & PLLDIV_DIV2 & OSCIOFNC_OFF & FCKSM_CSDCMD & POSCMOD_XT)
#else                   // PIC24FJ128GA010 and PIC24FJ256GA110
_CONFIG1( ICS_PGx2 & JTAGEN_OFF & GCP_OFF & GWRP_OFF & COE_OFF & FWDTEN_OFF) 
_CONFIG2( FNOSC_PRIPLL & OSCIOFNC_OFF & FCKSM_CSDCMD & POSCMOD_HS)
#endif

#define POT     5       // 10k potentiometer connected to AN5 input
#define TSENS   4       // TC1047 Temperature sensor with voltage output
#define AINPUTS 0xffcf  // Analog inputs for Explorer16 POT and TSENS


// initialize the ADC for single conversion, select Analog input pins
void initADC( int amask) 
{
    AD1PCFG = amask;    // select analog input pins
    AD1CON1 = 0x00E0;   // auto convert after end of sampling 
    AD1CSSL = 0;        // no scanning required 
    AD1CON3 = 0x1F02;   // max sample time = 31Tad, Tad = 2 x Tcy = 125ns >75ns
    AD1CON2 = 0;        // use MUXA, AVss and AVdd are used as Vref+/-
    AD1CON1bits.ADON = 1; // turn on the ADC
} //initADC


int readADC( int ch)
{
#if defined( __PIC24FJ256GB110__ ) || defined( __PIC24FJ256GA110__ )
    AD1CHS0  = ch;              // select analog input channel
#else
    AD1CHS  = ch;               // select analog input channel
#endif
    AD1CON1bits.SAMP = 1;       // start sampling, automatic conversion will follow
    while (!AD1CON1bits.DONE);   // wait to complete the conversion
    return ADC1BUF0;            // read the conversion result
} // readADC


main ()
{
    int a, r, c;
    
    // initializations
    initADC( AINPUTS);  // initialize the ADC for the Explorer16 analog inputs
    TRISA = 0xff00;     // select the PORTA pins as outputs to drive the LEDs
  	T1CON = 0x8000;     // TMR1 on, prescale 1:1 Tcy
    
    // use the first reading to randomize the number generator
    srand( readADC( POT)); 
    // generate the first random position
    r = 0x80 >> (rand() & 0x7); 
    // init the counter
    c = 0;
      
    // main loop
    while( 1)
    {   
        a = readADC( POT);  // select the POT input and convert 

        // reduce the 10-bit result to a 3 bit value (0..7)
        // (divide by 128 or shift right 7 times
        a >>= 7;
        
        // turn on only the corresponding LED
        // 0 -> leftmost LED.... 7-> rigtmost LED
        a = (0x80 >> a);

        // as soon as the cursor hits the food LED, generate a new one        
        while (a == r )
            r = 0x80 >> (rand() & 0x7);
          
        // display the user LED and 50% dim random LED
        if ((c & 0xf) == 0) 
            PORTA = a + r;
        else
            PORTA = a ;
            
        // counter to alternate loops
        c++;
    } // main loop
} // main
