/********1*********2*********3*********4*********5*********6*********7**********
*
* 	SDMMC read/write Test
*
*
*  History:
*   05/30/08 - Revised to checkout PIC24FJ256GB110/GA110 PIMs with AV32 board.
*              PIC24FJ256GB110/GA110 PIMs require use of ~/pps/pps.c. W Duquaine
*******************************************************************************/

#include <p24Fxxxx.h>
#include <string.h>

#include "SDMMC.h"


#if defined( __PIC24FJ256GB110__ ) || defined( __PIC24FJ256GB106__ )
#define PLL_96MHZ_ON    0xF7FF
_CONFIG1(ICS_PGx2 & JTAGEN_OFF & FWDTEN_OFF)        // JTAG off, watchdog timer off
_CONFIG2( FNOSC_PRIPLL & PLL_96MHZ_ON & PLLDIV_DIV2 & OSCIOFNC_OFF & FCKSM_CSDCMD & POSCMOD_XT)
#else                   // PIC24FJ128GA010 and PIC24FJ256GA110
_CONFIG1( ICS_PGx2 & JTAGEN_OFF & GCP_OFF & GWRP_OFF & COE_OFF & FWDTEN_OFF) 
_CONFIG2( FNOSC_PRIPLL & OSCIOFNC_OFF & FCKSM_CSDCMD & POSCMOD_HS)
#endif



void  Delayms ( unsigned t)
{
    T1CON = 0x8000;     // enable tmr1, Tcy, 1:1
    while (t--)
      {
        TMR1 = 0;
        while (TMR1<16000);
      }
} // Delayms


#define START_ADDRESS       10000   // start block address
#define N_BLOCKS            10000   // number of blocks
#define B_SIZE              512     // sector/data block size

char    data[ B_SIZE];
char  buffer[ B_SIZE];


main ( void)
{
    LBA addr;
    int i, r;

        // I/O initializations
    TRISA = 0xff00;		    // initialize PORTA LEDs output pins

    init_PPS ();                    // initialize PPS configuration for SPI2

    initSD();			    // initialize all I/Os required for the SD/MMC module

       // fill the buffer with "data"
    for ( i=0; i<B_SIZE; i++)
        data[i]= i;
 
       // wait for card to be inserted
    while ( !detectSD());   // assumes SDCD pin is by default an input
    Delayms ( 100);         // wait for card contacts de-bounce and power up
        
       // initialize the memory card (returns 0 if successful)
    r = initMedia();
    if ( r)                 // could not initialize the card
       {
         PORTA = r;         // show error code on LEDs
         while( 1);         // halt here
       }
    else
       {
           // fill N_BLOCK blocks/SECOTR with the contents of data buffer
        addr = START_ADDRESS;
        for( i=0; i<N_BLOCKS; i++)
            if (!writeSECTOR( addr+i, data))
               {   // writing failed
                 PORTA = 0x0f;
                 while( 1);  // halt here
               }

           // verify the contents of each block/SECTOR written
        addr = START_ADDRESS;
        for ( i=0; i<N_BLOCKS; i++)
          {   // read back one block at a time
            if (!readSECTOR( addr+i, buffer))
               {    // reading failed
                 PORTA = 0xf0;
                 while( 1);  // halt here
               }
            
               // verify each block content
 // WVD     if ( !memcmp( data, buffer, B_SIZE))  // This has to be a bug. Will == 0 if they both match !
            if (memcmp(data, buffer, B_SIZE) != 0)
               {    // block data mismatch
                 PORTA = 0x55;
                 while( 1); // halt here
               }
          } // for each block
       } // else media initialized

       // indicate successful execution
    PORTA = 0xFF;

       // main loop
    while( 1);

}      // main

