/*
** Audio PWM Test
**
** LDJ 7/10/06 v1.0
** WVD 8/30/08 - Add GB110/GA110 support
*/

#include <p24Fxxxx.h>

#define PLL_96MHZ_ON    0xF7FF
#if defined(__PIC24FJ256GB110__)     // GB110 requires PLLDIV etc, else clocks fail to turn on
_CONFIG1(ICS_PGx2 & JTAGEN_OFF & FWDTEN_OFF)        // JTAG off, watchdog timer off
_CONFIG2( FNOSC_PRIPLL & PLL_96MHZ_ON & PLLDIV_DIV2 & OSCIOFNC_OFF & FCKSM_CSDCMD & POSCMOD_XT)
#else                   // 128GA010 and 256GA110  PIMs
_CONFIG1( ICS_PGx2 & JTAGEN_OFF & GCP_OFF & GWRP_OFF & COE_OFF & FWDTEN_OFF) 
_CONFIG2( FNOSC_PRIPLL & OSCIOFNC_OFF & FCKSM_CSDCMD & POSCMOD_HS)
#endif

void  init_PPS(int use_spi1); // PPS support for GB110 and GA110 PIMs


void initAudio( long bitrate)
{
    // init the timebase
    T3CON = 0x8000;         // enable TMR3, prescale 1:1, internal clock
    PR3 = 400-1;            // set the period for the given bitrate
    _T3IF = 0;              // clear interrupt flag
    _T3IE = 1;              // enable TMR3 interrupt

    // init PWM
    // set the initial duty cycles
    OC1R = OC1RS = 200;  

    // activate the PWM modules 
#if defined(__PIC24FJ256GB110__)  ||  defined(__PIC24FJ256GA110__)
    OC1CON1 = 0x040E;       // CH1 and CH2 in PWM mode, TMR3 based
    OC1CON2 = 0x0000;       // no triggers, no faults used
#else                       // default GA010 PIM on Explorer16
    OC1CON = 0x000E;        // CH1 and CH2 in PWM mode, TMR3 based
#endif

} // initAudio


void haltAudio( void)
{
    T3CON = 0;              // disable TMR3
} // halt audio


void _ISRFAST _T3Interrupt( void)
{
    // clear interrupt flag and exit
    _T3IF = 0;
} // T3 Interrupt


main( void)
{
#if defined(__PIC24FJ256GB110__)  ||  defined(__PIC24FJ256GA110__)
    init_PPS (0);
#endif

    initAudio( 44100);
    
    // main loop
    while( 1);

}// main

