/*
** SPI2
*
* History:
*  05/18/08 - Revised to support PIC24FJ256GB110 and PIC24FJ256GA110. W Duquaine
*    Note that for GB110 we must use RG0 which was remapped to Explorer16 pin 79
*/

#include <p24Fxxxx.h>
#include "pps_exp16.h"

#define PLL_96MHZ_ON    0xF7FF
#if defined( __PIC24FJ256GB110__ ) || defined( __PIC24FJ256GB106__ )
_CONFIG1(ICS_PGx2 & JTAGEN_OFF & FWDTEN_OFF)        // JTAG off, watchdog timer off
_CONFIG2( FNOSC_PRIPLL & PLL_96MHZ_ON & PLLDIV_DIV2 & OSCIOFNC_OFF & FCKSM_CSDCMD & POSCMOD_XT)
#else                   // PIC24FJ128GA010 and PIC24FJ256GA110
_CONFIG1( ICS_PGx2 & JTAGEN_OFF & GCP_OFF & GWRP_OFF & COE_OFF & FWDTEN_OFF) 
_CONFIG2( FNOSC_PRIPLL & OSCIOFNC_OFF & FCKSM_CSDCMD & POSCMOD_HS)
#endif


// I/O definitions
//                 GB110 has its RG0 re-mapped to pin 79 on Explorer16
#if defined(__PIC24FJ256GB110__)
#define CSEE    _RG0        // select line (to pin 79) for Serial EEPROM
#define TCSEE   _TRISG0     // tris control for CSEE pin
#else
//                 GA010 and GA110 use standard RD12 to pin 79 on Explorer16
#define CSEE    _RD12       // select line (pin 79) for Serial EEPROM
#define TCSEE   _TRISD12    // tris control for CSEE pin
#endif

// peripheral configurations
#define SPI_MASTER  0x0120	// select 8-bit master mode, CKE=1, CKP =0
#define SPI_ENABLE	0x8000	// enable SPI port, clear status

// 25LC256 Serial EEPROM commands
#define SEE_WRSR	1		// write status register	
#define SEE_WRITE	2		// write command
#define SEE_READ	3		// read command
#define SEE_WDI		4 		// write disable
#define SEE_STAT	5		// read status register
#define SEE_WEN		6		// write enable


// send one byte of data and receive one back at the same time
int writeSPI2( int i)
{
    SPI2BUF = i;					// write to buffer for TX
    while( !SPI2STATbits.SPIRBF);	// wait for transfer to complete
    return SPI2BUF;    				// read the received value
}//writeSPI2


main()
{
	int i;

#if defined(__PIC24FJ256GB110__) || defined(__PIC24FJ256GA110__)
    init_PPS (0);           // initialize PPS configuration 
#endif

	// 1. init the SPI peripheral 
	TCSEE = 0;			    // make SSEE pin output
	CSEE  = 1;			    // de-select the Serial EEPROM
	SPI2CON1 = SPI_MASTER;  // select mode
	SPI2STAT = SPI_ENABLE; 	 // enable the peripheral

	// 2. main loop
	while( 1)
	  {	
    		// 2.1 send a Write Enable command
	    CSEE = 0;			    // select the Serial EEPROM
	    writeSPI2( SEE_WEN);	// write enable command
	    CSEE = 1;			    // deselect to complete the command
	    
	    	// 2.2 Check the Serial EEPROM status 
	    CSEE = 0;			    // select the Serial EEPROM
	    writeSPI2( SEE_STAT);  	// send a READ STATUS COMMAND
	    i = writeSPI2( 0);		// send/receive
	    CSEE = 1;		    	// deselect to terminate command
	  } // main loop
    
} // main
    
