/********1*********2*********3*********4*********5*********6*********7**********
*
* 	                          pps_exp16.c
* 
*
*  History:
*    05/30/08 - Added to allow PIC24FJ256GB110/GA110 PIMs to be used
*               with the AV32 board on Explorer16.                Wayne Duquaine
*******************************************************************************/

#include <p24Fxxxx.h>


#define SPI2_ENSURE_CLOCK_DONE  while(PORTGbits.RG6==1);   // GB110,GA110 Errata

              ///////////////////////////////////////////////
              //////    PPS Output Function Defines    //////
              ///////////////////////////////////////////////
#define U2TX_IO       5      // UART2 I/O on Explorer16
#define U2RTS_IO      6
#define SDO1_IO       7      // SPI1  I/O on Explorer16
#define SCK1OUT_IO    8
#define SDO2_IO      10      // SPI2  I/O on Explorer16
#define SCK2OUT_IO   11
#define SS2OUT_IO    12
#define OC1OUT_IO    18      // Output Compare I/O 
#define OC2OUT_IO    19
#define OC3OUT_IO    20
#define OC4OUT_IO    21



void  init_PPS (int use_sdi1)
{
#if defined(__PIC24FJ256GB110__) || defined(__PIC24FJ256GA110__)
        ////////////////////////////////////////////////////////////
        //   Do Peripheral Pin Select to setup UART2, SPI2,
        //   IC1, OC1, OC3, OC4 on PIC24 GB110 and GA110 PIMs
        ////////////////////////////////////////////////////////////
//  __builtin_write_OSCCONL (OSCCON & 0xDF); //clear IOLOCK to Unlock PPS config
            // Unlock the PPS configuration
    asm volatile ( "MOV   #OSCCON,w1 \n"
                   "MOV   #0x46,w2 \n"
                   "MOV   #0x57,w3 \n"
                   "MOV.b w2,[w1]  \n"
                   "MOV.b w3,[w1]  \n"
                   "BCLR  OSCCON,#6");
	
            // Map Input pins
    RPINR19bits.U2RXR  = 10;            // U2RX  = RP10   F4/pin 49
    RPINR19bits.U2CTSR = 32;            // U2CTS = RPI32  F12pin 40
    RPINR22bits.SDI2R  = 26;            // SDI2  = RP26   G7/pin 11
    RPINR7bits.IC1R    = 2;             // IC1   = RP2    D8/pin 68

            // Map Output pins
    RPOR8bits.RP17R  = U2TX_IO;         // U2TX  = RP17   F5/pin 50
    RPOR15bits.RP31R = U2RTS_IO;        // U2RTS = RP31   F13pin 39
    RPOR10bits.RP21R = SCK2OUT_IO;      // SCK2  = RP21   G6/pin 10
    RPOR9bits.RP19R  = SDO2_IO;         // SDO2  = RP19   G8/pin 12
    RPOR5bits.RP11R  = OC1OUT_IO;       // OC1   = RP11   D0/pin 72
    RPOR12bits.RP24R = OC2OUT_IO;       // OC2   = RP24   D1/pin 76
    RPOR11bits.RP22R = OC4OUT_IO;       // OC4   = RP22   D3/pin 78

            // GB110 shares usage of D2/pin 77 between SDI1 and OC3
            // On GB110 (USB PIM), pin 54 SDI1 is remapped to Explorer pin 77/D2
            //                     pin 55 SCK1 is remapped to Explorer pin 25/B0
            //                     pin 56 RG3  is remapped to Explorer pin 89/G1
            //                     pin 57 RG2  is remapped to Explorer pin 90/G0
    if (use_sdi1 == 0)
       {
         RPOR11bits.RP23R = OC3OUT_IO; // OC3  = RP23    D2/pin 77 (F7 on GA010)
       }
            /////////////////////////////////////////////////////////
            //  SPI1 is mapped differently between GB110 and GA110
            /////////////////////////////////////////////////////////
#if defined(__PIC24FJ256GB110__)
            // GB110 shares usage of D2/pin 77 between SDI1 and OC3
      else {
             RPINR20bits.SDI1R = 23;   // SDI1 = RP23    D2/pin 77 (F7 on GA010)
           }
    RPOR0bits.RP0R  = SCK1OUT_IO;      // SCK1 = RP0     B0/pin 25 (F6 on GA010) 
    RPOR7bits.RP15R = SDO1_IO;         // SDO1 = RP15    F8/pin 53
#else
            // GA110 does not share any pin usage between SDI1 and OC3
    RPINR20bits.SDI1R = 44;            // SDI1 = RPI44   F7/pin 54
//  RPOR0bits.RP0R  = SCK1OUT_IO;      // SCK1 = RPI45   F6/pin 55 INPUT ONLY SNAFU
//  RPINR20bits.SCK1R = 45;            // SCK1 = RPI45   F6/pin 55
    RPOR7bits.RP15R = SDO1_IO;         // SDO1 = RP15    F8/pin 53
#endif

//  __builtin_write_OSCCONL(OSCCON | 0x40);  // set IOLOCK to re-Lock PPS config
	    // re-Lock the PPS configuration
    asm volatile ( "mov #OSCCON,w1 \n"
                   "mov #0x46, w2 \n"
                   "mov #0x57, w3 \n"
                   "mov.b w2,[w1] \n"
                   "mov.b w3,[w1] \n"
                   "bset OSCCON, #6");
#else
            // PPS not required for PIC24FJ128GA010
#endif
}
